import os
from datetime import datetime
from functools import lru_cache
from pathlib import Path
from typing import Any

from ecoindex.models.enums import Grade
from pydantic import AnyHttpUrl, BaseModel, Field, field_validator

PageType = str


@lru_cache
def get_compute_version() -> str:
    current_directory = os.path.dirname(os.path.realpath(__file__))
    version_filename = os.path.join(current_directory, "..", "compute", "VERSION")

    with open(version_filename, "r") as f:
        return (f.read()).strip()


class Ecoindex(BaseModel):
    grade: Grade | None = Field(
        default=None,
        title="Ecoindex grade",
        description="Is the corresponding ecoindex grade of the page (from A to G)",
    )
    score: float | None = Field(
        default=None,
        title="Ecoindex score",
        description="Is the corresponding ecoindex score of the page (0 to 100)",
        ge=0,
        le=100,
    )
    ges: float | None = Field(
        default=None,
        title="Ecoindex GES equivalent",
        description=(
            "Is the equivalent of greenhouse gases emission" " (in `gCO2e`) of the page"
        ),
        ge=0,
    )
    water: float | None = Field(
        default=None,
        title="Ecoindex Water equivalent",
        description="Is the equivalent water consumption (in `cl`) of the page",
        ge=0,
    )
    ecoindex_version: str | None = Field(
        default=get_compute_version(),
        title="Ecoindex version",
        description="Is the version of the ecoindex used to compute the score",
    )


class PageMetrics(BaseModel):
    size: float = Field(
        default=...,
        title="Page size",
        description=(
            "Is the size of the page and of the downloaded"
            " elements of the page in KB"
        ),
        ge=0,
    )
    nodes: int = Field(
        default=...,
        title="Page nodes",
        description="Is the number of the DOM elements in the page",
        ge=0,
    )
    requests: int = Field(
        default=...,
        title="Page requests",
        description="Is the number of external requests made by the page",
        ge=0,
    )


class WebPage(BaseModel):
    width: int | None = Field(
        default=1920,
        title="Page Width",
        description="Width of the simulated window in pixel",
        ge=100,
        le=3840,
    )
    height: int | None = Field(
        default=1080,
        title="Page Height",
        description="Height of the simulated window in pixel",
        ge=50,
        le=2160,
    )
    url: str = Field(
        default=...,
        title="Page url",
        description="Url of the analysed page",
        examples=["https://www.ecoindex.fr"],
    )

    @field_validator("url")
    @classmethod
    def url_as_http_url(cls, v: str) -> str:
        url_object = AnyHttpUrl(url=v)
        assert url_object.scheme in {"http", "https"}, "scheme must be http or https"

        return url_object.unicode_string()

    def get_url_host(self) -> str:
        url_object = AnyHttpUrl(url=self.url)

        return str(url_object.host)

    def get_url_path(self) -> str:
        url_obect = AnyHttpUrl(url=self.url)

        return str(url_obect.path)


class WindowSize(BaseModel):
    height: int = Field(
        default=...,
        title="Window height",
        description="Height of the simulated window in pixel",
    )
    width: int = Field(
        default=...,
        title="Window width",
        description="Width of the simulated window in pixel",
    )

    def __str__(self) -> str:
        return f"{self.width},{self.height}"


class Result(Ecoindex, PageMetrics, WebPage):
    date: datetime | None = Field(
        default=None, title="Analysis datetime", description="Date of the analysis"
    )
    page_type: PageType | None = Field(
        default=None,
        title="Page type",
        description="Is the type of the page, based ton the [opengraph type tag](https://ogp.me/#types)",
    )


class ScreenShot(BaseModel):
    id: str
    folder: str

    def __init__(__pydantic_self__, **data: Any) -> None:
        super().__init__(**data)
        path = Path(__pydantic_self__.folder)
        path.mkdir(parents=True, exist_ok=True)

    def __str__(self) -> str:
        return f"{self.folder}/{self.id}"

    def get_png(self) -> str:
        return f"{self.__str__()}.png"

    def get_webp(self) -> str:
        return f"{self.__str__()}.webp"


class Request(BaseModel):
    url: str
    type: str
    size: float



from typing import Any

from ecoindex.models import Result
from pydantic import AnyHttpUrl, BaseModel, Field


class QueueTaskError(BaseModel):
    detail: Any | None = Field(
        default=None, title="Detail object of the raised exception"
    )
    exception: str = Field(default=..., title="Name of the exception that was raised")
    message: str = Field(default=..., title="Message of the exception")
    status_code: int | None = Field(
        default=None, title="Corresponding original HTTP status code sended by the API"
    )
    url: AnyHttpUrl | None = Field(default=None, title="URL of the analyzed web page")


class QueueTaskResult(BaseModel):
    status: str | None = Field(
        default=None,
        title="Status of the ecoindex analysis.",
        description=(
            "While the task is pending or the analysis is running, it is null."
            " But once the analysis is complete, it should return SUCCESS or FAILURE."
        ),
    )
    detail: Result | None = Field(
        default=None,
        title="Result of the ecoindex analysis once it was successfuly completed",
    )
    error: QueueTaskError | None = Field(
        default=None, title="Detail of the ecoindex error if it is not successful"
    )


class QueueTaskApi(BaseModel):
    id: str = Field(
        default=...,
        title=(
            "Identifier of the current. "
            "This identifier will become the identifier of the analysis"
        ),
    )
    status: str = Field(
        default=...,
        title="Status of the current task. Can be PENDING, FAILURE, SUCCESS",
    )
    ecoindex_result: QueueTaskResult | None = Field(
        default=None, title="Result of the Ecoindex analysis"
    )
    task_error: Any | None = Field(
        default=None,
        title="Detail of the error encountered by the task in case of Failure",
    )



from pydantic import BaseModel


class RequestItem(BaseModel):
    category: str
    mime_type: str
    size: float
    status: int
    url: str


class MimetypeMetrics(BaseModel):
    total_count: int = 0
    total_size: float = 0


class MimetypeAggregation(BaseModel):
    audio: MimetypeMetrics = MimetypeMetrics()
    css: MimetypeMetrics = MimetypeMetrics()
    font: MimetypeMetrics = MimetypeMetrics()
    html: MimetypeMetrics = MimetypeMetrics()
    image: MimetypeMetrics = MimetypeMetrics()
    javascript: MimetypeMetrics = MimetypeMetrics()
    other: MimetypeMetrics = MimetypeMetrics()
    video: MimetypeMetrics = MimetypeMetrics()

    @classmethod
    async def get_category_of_resource(cls, mimetype: str) -> str:
        mimetypes = [type for type in cls.model_fields.keys()]

        for type in mimetypes:
            if type in mimetype:
                return type

        return "other"


class Requests(BaseModel):
    aggregation: MimetypeAggregation = MimetypeAggregation()
    items: list[RequestItem] = []
    total_count: int = 0
    total_size: float = 0



from ecoindex.data import (  # noqa: F401
    A,
    B,
    C,
    D,
    E,
    F,
    G,
    quantiles_dom,
    quantiles_req,
    quantiles_size,
)
from ecoindex.models import Ecoindex
from typing_extensions import deprecated


async def get_quantile(quantiles: list[int | float], value: int | float) -> float:
    for i in range(1, len(quantiles)):
        if value < quantiles[i]:
            return (
                i - 1 + (value - quantiles[i - 1]) / (quantiles[i] - quantiles[i - 1])
            )

    return len(quantiles) - 1


async def get_score(dom: int, size: float, requests: int) -> float:
    q_dom = await get_quantile(quantiles_dom, dom)  # type: ignore
    q_size = await get_quantile(quantiles_size, size)
    q_req = await get_quantile(quantiles_req, requests)  # type: ignore

    return round(100 - 5 * (3 * q_dom + 2 * q_req + q_size) / 6)


@deprecated("Use compute_ecoindex instead")
async def get_ecoindex(dom: int, size: float, requests: int) -> Ecoindex:
    score = await get_score(dom=dom, size=size, requests=requests)

    return Ecoindex(
        score=score,
        grade=await get_grade(score),
        ges=await get_greenhouse_gases_emmission(score),
        water=await get_water_consumption(score),
    )


async def compute_ecoindex(nodes: int, size: float, requests: int) -> Ecoindex:
    return await get_ecoindex(
        dom=nodes,
        size=size,
        requests=requests,
    )


async def get_grade(ecoindex: float) -> str:
    for grade in "ABCDEF":
        if ecoindex > globals()[grade]:
            return grade

    return "G"


async def get_greenhouse_gases_emmission(ecoindex: float) -> float:
    return round(100 * (2 + 2 * (50 - ecoindex) / 100)) / 100


async def get_water_consumption(ecoindex: float) -> float:
    return round(100 * (3 + 3 * (50 - ecoindex) / 100)) / 100



from typing import Any

from pydantic import BaseModel, Field


class ApiHealth(BaseModel):
    database: bool = Field(default=..., title="Status of database")


class BaseHost(BaseModel):
    name: str
    total_count: int


class Host(BaseHost):
    remaining_daily_requests: int | None = None


class PageHosts(BaseModel):
    items: list[str]
    total: int
    page: int
    size: int


class ExceptionResponse(BaseModel):
    args: list[Any]
    exception: str
    message: str | None = None


class HealthWorker(BaseModel):
    name: str = Field(default=..., title="Name of worker")
    healthy: bool = Field(default=..., title="Status of worker")


class HealthWorkers(BaseModel):
    healthy: bool = Field(default=..., title="Global status of workers")
    workers: list[HealthWorker] = Field(default=..., title="List of workers")


class HealthResponse(BaseModel):
    database: bool = Field(default=..., title="Status of database")
    workers: HealthWorkers = Field(default=..., title="Status of workers")



from enum import Enum


class Version(str, Enum):
    v0 = "v0"
    v1 = "v1"

    def get_version_number(self) -> int:
        return int(self.value[1:])


class ExportFormat(Enum):
    csv = "csv"
    json = "json"


class Language(Enum):
    fr = "fr"
    en = "en"


class TaskStatus(str, Enum):
    FAILURE = "FAILURE"
    PENDING = "PENDING"
    SUCCESS = "SUCCESS"


class BadgeTheme(str, Enum):
    dark = "dark"
    light = "light"


class Grade(str, Enum):
    A = "A"
    B = "B"
    C = "C"
    D = "D"
    E = "E"
    F = "F"
    G = "G"



from ecoindex.data.colors import A as A_color
from ecoindex.data.colors import B as B_color
from ecoindex.data.colors import C as C_color
from ecoindex.data.colors import D as D_color
from ecoindex.data.colors import E as E_color
from ecoindex.data.colors import F as F_color
from ecoindex.data.colors import G as G_color
from ecoindex.data.grades import A, B, C, D, E, F, G
from ecoindex.data.medians import (
    median_dom,
    median_req,
    median_size,
)
from ecoindex.data.quantiles import (
    quantiles_dom,
    quantiles_req,
    quantiles_size,
)
from ecoindex.data.targets import (
    target_dom,
    target_req,
    target_size,
)

__all__ = [
    "A",
    "B",
    "C",
    "D",
    "E",
    "F",
    "G",
    "A_color",
    "B_color",
    "C_color",
    "D_color",
    "E_color",
    "F_color",
    "G_color",
    "median_dom",
    "median_req",
    "median_size",
    "quantiles_dom",
    "quantiles_req",
    "quantiles_size",
    "target_dom",
    "target_req",
    "target_size",
]
